/* ====================================
		bZ[W
==================================== */
#include "main.h"


/* ====================================
		錾
==================================== */
typedef BOOL (*MSG_FUNC)(HWND, WPARAM, LPARAM, LRESULT &);
typedef BOOL (*CTRL_FUNC)(HWND, HWND, WORD, WORD, LRESULT &);

typedef struct {
	UINT     msgId;
	MSG_FUNC msgFunc;
} MSGTABLE, *LPMSGTABLE;

typedef struct {
	UINT      ctrlId;
	CTRL_FUNC ctrlFunc;
} CTRLTABLE, *LPCTRLTABLE;

static BOOL OnCommand(HWND hwnd, WPARAM wParam, LPARAM lParam, LRESULT &result);


/* ====================================
		ϐ
==================================== */
static const TCHAR INI_NAME[] = TEXT("gdi++.ini");

static UINT           g_taskbarRestart = 0;	// ^XNo[̍č\̃bZ[W
static HMENU          g_hMenu = NULL;
static NOTIFYICONDATA g_nIcon;


/* ====================================
		`
==================================== */
// Destroy
static BOOL OnDestroy(HWND hwnd, WPARAM wParam, LPARAM lParam, LRESULT &result)
{
	TerminateHook();
	Shell_NotifyIcon(NIM_DELETE, &g_nIcon);
	PostQuitMessage(0);
	return TRUE;
}

// Create
static BOOL OnCreate(HWND hwnd, WPARAM wParam, LPARAM lParam, LRESULT &result)
{
	g_taskbarRestart = RegisterWindowMessage(TEXT("TaskbarCreated"));
	g_hMenu          = LoadMenu(((LPCREATESTRUCT)(lParam))->hInstance, MAKEINTRESOURCE(IDM_MNU));

	if (g_taskbarRestart && g_hMenu) {
		if (SetTaskTrayIcon(hwnd, &g_nIcon))
			return TRUE;
		MessageBox(NULL, TEXT("Failure SetTaskTrayIcon"), NULL, MB_ICONSTOP);
	} else  {
		if (!g_hMenu)
			MessageBox(NULL, TEXT("Failure LoadMenu"), NULL, MB_ICONSTOP);
		else
			MessageBox(NULL, TEXT("Failure RegisterWindowMessage"), NULL, MB_ICONSTOP);
	}
	result = -1;
	return TRUE;
}

// TaskTray
static BOOL OnTaskTray(HWND hwnd, WPARAM wParam, LPARAM lParam, LRESULT &result)
{
	if (wParam != ID_TRAY) { return FALSE; }

	switch (lParam) {
	// j[\
	case WM_RBUTTONUP:
		{
			POINT point;
			GetCursorPos(&point);
			SetForegroundWindow(hwnd);

			UINT flag = (lstrcmp(g_dllPath, TEXT("")) ? MF_ENABLED : MF_GRAYED);
			EnableMenuItem(g_hMenu, IDM_OPENINI, MF_BYCOMMAND | flag);
			EnableMenuItem(g_hMenu, IDM_OPENDIR, MF_BYCOMMAND | flag);
			CheckMenuItem(g_hMenu, IDM_ENABLED, (g_hHook) ? MF_CHECKED : MF_UNCHECKED);
			TrackPopupMenu(GetSubMenu(g_hMenu, 0),
				TPM_TOPALIGN | TPM_LEFTALIGN, point.x, point.y, 0, hwnd, NULL);
		}
		break;

	// OnMenuHook
	case WM_LBUTTONDBLCLK:
		SendMessage(hwnd, WM_COMMAND, MAKEWPARAM(IDM_ENABLED, 0), (LPARAM)GetDlgItem(hwnd, IDM_ENABLED));
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

// Command - Menu - OpenIni
static BOOL OnMenuOpenIni(HWND hwnd, HWND hCtrl, WORD wID, WORD wNotify, LRESULT &result)
{
	TCHAR iniPath[MAX_PATH];
	lstrcpy(iniPath, g_dllPath);
	del_title(iniPath);
	add_title(iniPath, INI_NAME);
	ShellExec(hwnd, iniPath);
	return TRUE;
}

// Command - Menu - OpenDir
static BOOL OnMenuOpenDir(HWND hwnd, HWND hCtrl, WORD wID, WORD wNotify, LRESULT &result)
{
	TCHAR dllDir[MAX_PATH];
	lstrcpy(dllDir, g_dllPath);
	del_title(dllDir);
	ShellExec(hwnd, dllDir);
	return TRUE;
}

// Command - Menu - Hook
static BOOL OnMenuHook(HWND hwnd, HWND hCtrl, WORD wID, WORD wNotify, LRESULT &result)
{
	if (g_hHook == NULL) {
		InitializeHook();
	} else {
		TerminateHook();
	}
	return TRUE;
}

// Command - Menu - VerInfo
static BOOL OnMenuVerInfo(HWND hwnd, HWND hCtrl, WORD wID, WORD wNotify, LRESULT &result)
{
	if (DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_DLG1001), hwnd, DialogProc) == IDABORT) {
		MessageBox(NULL, TEXT("Failure OnMenuVerInfo(OnInitDialog)"), NULL, MB_ICONSTOP);
		DestroyWindow(hwnd);
	}
	return TRUE;
}

// Command - Menu - Exit
static BOOL OnMenuExit(HWND hwnd, HWND hCtrl, WORD wID, WORD wNotify, LRESULT &result)
{
	DestroyWindow(hwnd);
	return TRUE;
}


/* ====================================
		}bv
==================================== */
// MainMsg
static const MSGTABLE msgHandler[] = {
	{WM_TRAYICONMESSAGE , OnTaskTray},
	{WM_DESTROY         , OnDestroy},
	{WM_CREATE          , OnCreate},
	{WM_COMMAND         , OnCommand},
};

// MainMsg - Command
static const CTRLTABLE ctrlHandler[] = {
	{IDM_OPENINI , OnMenuOpenIni},
	{IDM_OPENDIR , OnMenuOpenDir},
	{IDM_ENABLED , OnMenuHook},
	{IDM_VERINFO , OnMenuVerInfo},
	{IDM_EXIT    , OnMenuExit},
};


/* ====================================
		C^[tF[X
==================================== */
// MainMsgProc
LRESULT CALLBACK WndMainProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	LRESULT    result   = 0;
	static int tableLen = sizeof(msgHandler) / sizeof(msgHandler[0]);

	for (int i=0 ; i<tableLen ; i++) {
		if (msgHandler[i].msgId != msg) { continue; }
		if((msgHandler[i].msgFunc) && (msgHandler[i].msgFunc(hwnd, wParam, lParam, result))) {
			return result;
		}
	}

	// 萔bZ[W
	if (msg == g_taskbarRestart) {
		if (!SetTaskTrayIcon(hwnd, &g_nIcon)) {
			MessageBox(NULL, TEXT("Failure SetTaskTrayIcon"), NULL, MB_ICONSTOP);
			DestroyWindow(hwnd);
		}
		return 0;
	}
	return DefWindowProc(hwnd, msg, wParam, lParam);
}

// MainMsgProc - Command
static BOOL OnCommand(HWND hwnd, WPARAM wParam, LPARAM lParam, LRESULT &result)
{
	UINT       ctrl     = LOWORD(wParam);
	static int tableLen = sizeof(ctrlHandler) / sizeof(ctrlHandler[0]);

	for (int i=0 ; i<tableLen ; i++) {
		if ((ctrlHandler[i].ctrlId == ctrl) && (ctrlHandler[i].ctrlFunc)) {
			return ctrlHandler[i].ctrlFunc(hwnd, (HWND)lParam, LOWORD(wParam), HIWORD(wParam), result);
		}
	}
	return FALSE;
}

// EOF
